@@ -40,13 +40,16 @@
  * <title>Using an element factory</title>
  * <programlisting language="c">
  *   #include &lt;gst/gst.h&gt;
+ *
  *   GstElement *src;
  *   GstElementFactory *srcfactory;
- *   gst_init(&amp;argc,&amp;argv);
- *   srcfactory = gst_element_factory_find("filesrc");
- *   g_return_if_fail(srcfactory != NULL);
- *   src = gst_element_factory_create(srcfactory,"src");
- *   g_return_if_fail(src != NULL);
+ *
+ *   gst_init (&amp;argc, &amp;argv);
+ *
+ *   srcfactory = gst_element_factory_find ("filesrc");
+ *   g_return_if_fail (srcfactory != NULL);
+ *   src = gst_element_factory_create (srcfactory, "src");
+ *   g_return_if_fail (src != NULL);
  *   ...
  * </programlisting>
  * </example>
@@ -61,6 +64,8 @@
 #include "gsturi.h"
 #include "gstregistry.h"
 
+#include "glib-compat-private.h"
+
 GST_DEBUG_CATEGORY_STATIC (element_factory_debug);
 #define GST_CAT_DEFAULT element_factory_debug
 
@@ -74,47 +79,27 @@ static GstPluginFeatureClass *parent_class = NULL;
 
 /* static guint gst_element_factory_signals[LAST_SIGNAL] = { 0 }; */
 
-GType
-gst_element_factory_get_type (void)
-{
-  static GType elementfactory_type = 0;
+/* this is defined in gstelement.c */
+extern GQuark _gst_elementclass_factory;
 
-  if (G_UNLIKELY (elementfactory_type == 0)) {
-    static const GTypeInfo elementfactory_info = {
-      sizeof (GstElementFactoryClass),
-      NULL,
-      NULL,
-      (GClassInitFunc) gst_element_factory_class_init,
-      NULL,
-      NULL,
-      sizeof (GstElementFactory),
-      0,
-      (GInstanceInitFunc) gst_element_factory_init,
-      NULL
-    };
-
-    elementfactory_type = g_type_register_static (GST_TYPE_PLUGIN_FEATURE,
-        "GstElementFactory", &elementfactory_info, 0);
-    GST_DEBUG_CATEGORY_INIT (element_factory_debug, "GST_ELEMENT_FACTORY",
-        GST_DEBUG_BOLD | GST_DEBUG_FG_WHITE | GST_DEBUG_BG_RED,
-        "element factories keep information about installed elements");
-  }
-  return elementfactory_type;
+#define _do_init \
+{ \
+  GST_DEBUG_CATEGORY_INIT (element_factory_debug, "GST_ELEMENT_FACTORY", \
+      GST_DEBUG_BOLD | GST_DEBUG_FG_WHITE | GST_DEBUG_BG_RED, \
+      "element factories keep information about installed elements"); \
 }
+
+G_DEFINE_TYPE_WITH_CODE (GstElementFactory, gst_element_factory,
+    GST_TYPE_PLUGIN_FEATURE, _do_init);
+
 static void
 gst_element_factory_class_init (GstElementFactoryClass * klass)
 {
-  GObjectClass *gobject_class;
-  GstObjectClass *gstobject_class;
-  GstPluginFeatureClass *gstpluginfeature_class;
-
-  gobject_class = (GObjectClass *) klass;
-  gstobject_class = (GstObjectClass *) klass;
-  gstpluginfeature_class = (GstPluginFeatureClass *) klass;
+  GObjectClass *gobject_class = (GObjectClass *) klass;
 
   parent_class = g_type_class_peek_parent (klass);
 
-  gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_element_factory_finalize);
+  gobject_class->finalize = gst_element_factory_finalize;
 }
 
 static void
@@ -169,13 +154,10 @@ void
 __gst_element_details_clear (GstElementDetails * dp)
 {
   g_free (dp->longname);
-  dp->longname = NULL;
   g_free (dp->klass);
-  dp->klass = NULL;
   g_free (dp->description);
-  dp->description = NULL;
   g_free (dp->author);
-  dp->author = NULL;
+  memset (dp, 0, sizeof (GstElementDetails));
 }
 
 #define VALIDATE_SET(__dest, __src, __entry)                            \
@@ -214,16 +196,28 @@ gst_element_factory_cleanup (GstElementFactory * factory)
 
   __gst_element_details_clear (&factory->details);
   if (factory->type) {
-    g_type_class_unref (g_type_class_peek (factory->type));
-    factory->type = 0;
+    factory->type = G_TYPE_INVALID;
   }
 
   for (item = factory->staticpadtemplates; item; item = item->next) {
     GstStaticPadTemplate *templ = item->data;
+    GstCaps *caps = (GstCaps *) & (templ->static_caps);
 
-    g_free (templ->name_template);
     g_free ((gchar *) templ->static_caps.string);
-    memset (&(templ->static_caps), 0, sizeof (GstStaticCaps));
+
+    /* FIXME: this is not threadsafe */
+    if (caps->refcount == 1) {
+      GstStructure *structure;
+      guint i;
+
+      for (i = 0; i < caps->structs->len; i++) {
+        structure = (GstStructure *) gst_caps_get_structure (caps, i);
+        gst_structure_set_parent_refcount (structure, NULL);
+        gst_structure_free (structure);
+      }
+      g_ptr_array_free (caps->structs, TRUE);
+      caps->refcount = 0;
+    }
     g_free (templ);
   }
   g_list_free (factory->staticpadtemplates);
@@ -242,7 +236,8 @@ gst_element_factory_cleanup (GstElementFactory * factory)
 
 /**
  * gst_element_register:
- * @plugin: #GstPlugin to register the element with
+ * @plugin: #GstPlugin to register the element with, or NULL for a static
+ * element (note that passing NULL only works in GStreamer 0.10.13 and later)
  * @name: name of elements of this type
  * @rank: rank of element (higher rank means more importance when autoplugging)
  * @type: GType of element to register
@@ -256,6 +251,8 @@ gboolean
 gst_element_register (GstPlugin * plugin, const gchar * name, guint rank,
     GType type)
 {
+  GstPluginFeature *existing_feature;
+  GstRegistry *registry;
   GstElementFactory *factory;
   GType *interfaces;
   guint n_interfaces, i;
@@ -265,11 +262,33 @@ gst_element_register (GstPlugin * plugin, const gchar * name, guint rank,
   g_return_val_if_fail (name != NULL, FALSE);
   g_return_val_if_fail (g_type_is_a (type, GST_TYPE_ELEMENT), FALSE);
 
-  factory = GST_ELEMENT_FACTORY (g_object_new (GST_TYPE_ELEMENT_FACTORY, NULL));
-  gst_plugin_feature_set_name (GST_PLUGIN_FEATURE (factory), name);
+  registry = gst_registry_get_default ();
+
+  /* check if feature already exists, if it exists there is no need to update it
+   * when the registry is getting updated, outdated plugins and all their
+   * features are removed and readded.
+   */
+  existing_feature = gst_registry_lookup_feature (registry, name);
+  if (existing_feature) {
+    GST_DEBUG_OBJECT (registry, "update existing feature %p (%s)",
+        existing_feature, name);
+    factory = GST_ELEMENT_FACTORY_CAST (existing_feature);
+    factory->type = type;
+    existing_feature->loaded = TRUE;
+    g_type_set_qdata (type, _gst_elementclass_factory, factory);
+    gst_object_unref (existing_feature);
+    return TRUE;
+  }
+
+  factory =
+      GST_ELEMENT_FACTORY_CAST (g_object_newv (GST_TYPE_ELEMENT_FACTORY, 0,
+          NULL));
+  gst_plugin_feature_set_name (GST_PLUGIN_FEATURE_CAST (factory), name);
   GST_LOG_OBJECT (factory, "Created new elementfactory for type %s",
       g_type_name (type));
 
+  /* provide info needed during class structure setup */
+  g_type_set_qdata (type, _gst_elementclass_factory, factory);
   klass = GST_ELEMENT_CLASS (g_type_class_ref (type));
   if ((klass->details.longname == NULL) ||
       (klass->details.klass == NULL) || (klass->details.author == NULL))
@@ -282,7 +301,7 @@ gst_element_register (GstPlugin * plugin, const gchar * name, guint rank,
     GstStaticPadTemplate *newt;
 
     newt = g_new0 (GstStaticPadTemplate, 1);
-    newt->name_template = g_strdup (templ->name_template);
+    newt->name_template = g_intern_string (templ->name_template);
     newt->direction = templ->direction;
     newt->presence = templ->presence;
     newt->static_caps.string = gst_caps_to_string (templ->caps);
@@ -290,19 +309,25 @@ gst_element_register (GstPlugin * plugin, const gchar * name, guint rank,
         g_list_append (factory->staticpadtemplates, newt);
   }
   factory->numpadtemplates = klass->numpadtemplates;
-  klass->elementfactory = factory;
 
   /* special stuff for URI handling */
   if (g_type_is_a (type, GST_TYPE_URI_HANDLER)) {
     GstURIHandlerInterface *iface = (GstURIHandlerInterface *)
         g_type_interface_peek (klass, GST_TYPE_URI_HANDLER);
 
-    if (!iface || !iface->get_type || !iface->get_protocols)
+    if (!iface || (!iface->get_type && !iface->get_type_full) ||
+        (!iface->get_protocols && !iface->get_protocols_full))
       goto urierror;
-    factory->uri_type = iface->get_type ();
+    if (iface->get_type)
+      factory->uri_type = iface->get_type ();
+    else if (iface->get_type_full)
+      factory->uri_type = iface->get_type_full (factory->type);
     if (!GST_URI_TYPE_IS_VALID (factory->uri_type))
       goto urierror;
-    factory->uri_protocols = g_strdupv (iface->get_protocols ());
+    if (iface->get_protocols)
+      factory->uri_protocols = g_strdupv (iface->get_protocols ());
+    else if (iface->get_protocols_full)
+      factory->uri_protocols = iface->get_protocols_full (factory->type);
     if (!factory->uri_protocols)
       goto urierror;
   }
@@ -313,12 +338,15 @@ gst_element_register (GstPlugin * plugin, const gchar * name, guint rank,
   }
   g_free (interfaces);
 
-  GST_PLUGIN_FEATURE (factory)->plugin_name = g_strdup (plugin->desc.name);
-  gst_plugin_feature_set_rank (GST_PLUGIN_FEATURE (factory), rank);
-  GST_PLUGIN_FEATURE (factory)->loaded = TRUE;
+  if (plugin && plugin->desc.name) {
+    GST_PLUGIN_FEATURE_CAST (factory)->plugin_name = plugin->desc.name;
+  } else {
+    GST_PLUGIN_FEATURE_CAST (factory)->plugin_name = "NULL";
+  }
+  gst_plugin_feature_set_rank (GST_PLUGIN_FEATURE_CAST (factory), rank);
+  GST_PLUGIN_FEATURE_CAST (factory)->loaded = TRUE;
 
-  gst_registry_add_feature (gst_registry_get_default (),
-      GST_PLUGIN_FEATURE (factory));
+  gst_registry_add_feature (registry, GST_PLUGIN_FEATURE_CAST (factory));
 
   return TRUE;
 
@@ -377,24 +405,27 @@ gst_element_factory_create (GstElementFactory * factory, const gchar * name)
   if (factory->type == 0)
     goto no_type;
 
-  /* create an instance of the element, cast so we don't assert on NULL */
-  element = GST_ELEMENT_CAST (g_object_new (factory->type, NULL));
+  /* create an instance of the element, cast so we don't assert on NULL
+   * also set name as early as we can
+   */
+  if (name)
+    element =
+        GST_ELEMENT_CAST (g_object_new (factory->type, "name", name, NULL));
+  else
+    element = GST_ELEMENT_CAST (g_object_newv (factory->type, 0, NULL));
   if (G_UNLIKELY (element == NULL))
     goto no_element;
 
   /* fill in the pointer to the factory in the element class. The
-   * class will not be unreffed currently. 
-   * FIXME: This isn't safe and may leak a refcount on the factory if 2 threads
-   * create the first instance of an element at the same moment */
+   * class will not be unreffed currently.
+   * Be thread safe as there might be 2 threads creating the first instance of
+   * an element at the same moment
+   */
   oclass = GST_ELEMENT_GET_CLASS (element);
-  if (G_UNLIKELY (oclass->elementfactory == NULL))
-    oclass->elementfactory = factory;
-  else
+  if (!g_atomic_pointer_compare_and_exchange (
+          (gpointer) & oclass->elementfactory, NULL, factory))
     gst_object_unref (factory);
 
-  if (name)
-    gst_object_set_name (GST_OBJECT (element), name);
-
   GST_DEBUG ("created element \"%s\"", GST_PLUGIN_FEATURE_NAME (factory));
 
   return element;
@@ -402,7 +433,8 @@ gst_element_factory_create (GstElementFactory * factory, const gchar * name)
   /* ERRORS */
 load_failed:
   {
-    GST_WARNING_OBJECT (factory, "loading plugin returned NULL!");
+    GST_WARNING_OBJECT (factory,
+        "loading plugin containing feature %s returned NULL!", name);
     return NULL;
   }
 no_type:
@@ -448,10 +480,10 @@ gst_element_factory_make (const gchar * factoryname, const gchar * name)
 
   GST_LOG_OBJECT (factory, "found factory %p", factory);
   element = gst_element_factory_create (factory, name);
-  gst_object_unref (factory);
   if (element == NULL)
     goto create_failed;
 
+  gst_object_unref (factory);
   return element;
 
   /* ERRORS */
@@ -463,6 +495,7 @@ no_factory:
 create_failed:
   {
     GST_INFO_OBJECT (factory, "couldn't create instance!");
+    gst_object_unref (factory);
     return NULL;
   }
 }
@@ -618,7 +651,7 @@ gst_element_factory_get_static_pad_templates (GstElementFactory * factory)
  * gst_element_factory_get_uri_type:
  * @factory: a #GstElementFactory
  *
- * Gets the type of URIs the element supports or GST_URI_UNKNOWN if none.
+ * Gets the type of URIs the element supports or #GST_URI_UNKNOWN if none.
  *
  * Returns: type of URIs this element supports
  */
@@ -648,3 +681,31 @@ gst_element_factory_get_uri_protocols (GstElementFactory * factory)
 
   return factory->uri_protocols;
 }
+
+/**
+ * gst_element_factory_has_interface:
+ * @factory: a #GstElementFactory
+ * @interfacename: an interface name
+ *
+ * Check if @factory implements the interface with name @interfacename.
+ *
+ * Returns: #TRUE when @factory implement the interface.
+ *
+ * Since: 0.10.14
+ */
+gboolean
+gst_element_factory_has_interface (GstElementFactory * factory,
+    const gchar * interfacename)
+{
+  GList *walk;
+
+  g_return_val_if_fail (GST_IS_ELEMENT_FACTORY (factory), FALSE);
+
+  for (walk = factory->interfaces; walk; walk = g_list_next (walk)) {
+    gchar *iname = (gchar *) walk->data;
+
+    if (!strcmp (iname, interfacename))
+      return TRUE;
+  }
+  return FALSE;
+}
