@@ -27,8 +27,8 @@
  * @see_also: #GstElement, #GstBin, #GstClock, #GstBus
  *
  * A #GstPipeline is a special #GstBin used as the toplevel container for
- * the filter graph. The #GstPipeline will manage the selection and 
- * distribution of a global #GstClock as well as provide a #GstBus to the 
+ * the filter graph. The #GstPipeline will manage the selection and
+ * distribution of a global #GstClock as well as provide a #GstBus to the
  * application. It will also implement a default behavour for managing
  * seek events (see gst_element_seek()).
  *
@@ -36,7 +36,7 @@
  * the pipeline, use gst_object_unref() to free its resources including all
  * added #GstElement objects (if not otherwise referenced).
  *
- * Elements are added and removed from the pipeline using the #GstBin 
+ * Elements are added and removed from the pipeline using the #GstBin
  * methods like gst_bin_add() and gst_bin_remove() (see #GstBin).
  *
  * Before changing the state of the #GstPipeline (see #GstElement) a #GstBus
@@ -50,8 +50,8 @@
  *
  * When the #GstPipeline performs the PAUSED to PLAYING state change it will
  * select a clock for the elements. The clock selection algorithm will by
- * default select a clock provided by an element that is most upstream 
- * (closest to the source). For live pipelines (ones that return 
+ * default select a clock provided by an element that is most upstream
+ * (closest to the source). For live pipelines (ones that return
  * #GST_STATE_CHANGE_NO_PREROLL from the gst_element_set_state() call) this
  * will select the clock provided by the live source. For normal pipelines
  * this will select a clock provided by the sinks (most likely the audio
@@ -59,24 +59,26 @@
  *
  * The clock selection can be controlled with the gst_pipeline_use_clock()
  * method, which will enforce a given clock on the pipeline. With
- * gst_pipeline_auto_clock() the default clock selection algorithm can be 
+ * gst_pipeline_auto_clock() the default clock selection algorithm can be
  * restored.
  *
- * A #GstPipeline maintains a stream time for the elements. The stream
+ * A #GstPipeline maintains a running time for the elements. The running
  * time is defined as the difference between the current clock time and
  * the base time. When the pipeline goes to READY or a flushing seek is
- * performed on it, the stream time is reset to 0. When the pipeline is
- * set from PLAYING to PAUSED, the current clock time is sampled and used to 
+ * performed on it, the running time is reset to 0. When the pipeline is
+ * set from PLAYING to PAUSED, the current clock time is sampled and used to
  * configure the base time for the elements when the pipeline is set
- * to PLAYING again. This default behaviour can be changed with the
- * gst_pipeline_set_new_stream_time() method. 
- * 
- * When sending a flushing seek event to a GstPipeline (see 
- * gst_element_seek()), it will make sure that the pipeline is properly 
- * PAUSED and resumed as well as set the new stream time to 0 when the
+ * to PLAYING again. The effect is that the running time (as the difference
+ * between the clock time and the base time) will count how much time was spent
+ * in the PLAYING state. This default behaviour can be changed with the
+ * gst_element_set_start_time() method.
+ *
+ * When sending a flushing seek event to a GstPipeline (see
+ * gst_element_seek()), it will make sure that the pipeline is properly
+ * PAUSED and resumed as well as set the new running time to 0 when the
  * seek succeeded.
  *
- * Last reviewed on 2006-03-12 (0.10.5)
+ * Last reviewed on 2009-05-29 (0.10.24)
  */
 
 #include "gst_private.h"
@@ -86,16 +88,11 @@
 #include "gstpipeline.h"
 #include "gstinfo.h"
 #include "gstsystemclock.h"
+#include "gstutils.h"
 
 GST_DEBUG_CATEGORY_STATIC (pipeline_debug);
 #define GST_CAT_DEFAULT pipeline_debug
 
-static const GstElementDetails gst_pipeline_details =
-GST_ELEMENT_DETAILS ("Pipeline object",
-    "Generic/Bin",
-    "Complete pipeline object",
-    "Erik Walthinsen <omega@cse.ogi.edu>, Wim Taymans <wim@fluendo.com>");
-
 /* Pipeline signals and args */
 enum
 {
@@ -120,12 +117,15 @@ struct _GstPipelinePrivate
 {
   /* with LOCK */
   gboolean auto_flush_bus;
+
+  /* when we need to update stream_time or clock when going back to
+   * PLAYING*/
+  GstClockTime last_start_time;
+  gboolean update_clock;
 };
 
 
 static void gst_pipeline_base_init (gpointer g_class);
-static void gst_pipeline_class_init (gpointer g_class, gpointer class_data);
-static void gst_pipeline_init (GTypeInstance * instance, gpointer g_class);
 
 static void gst_pipeline_dispose (GObject * object);
 static void gst_pipeline_set_property (GObject * object, guint prop_id,
@@ -133,66 +133,45 @@ static void gst_pipeline_set_property (GObject * object, guint prop_id,
 static void gst_pipeline_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
 
-static gboolean gst_pipeline_send_event (GstElement * element,
-    GstEvent * event);
-
 static GstClock *gst_pipeline_provide_clock_func (GstElement * element);
 static GstStateChangeReturn gst_pipeline_change_state (GstElement * element,
     GstStateChange transition);
 
-static GstBinClass *parent_class = NULL;
+static void gst_pipeline_handle_message (GstBin * bin, GstMessage * message);
 
 /* static guint gst_pipeline_signals[LAST_SIGNAL] = { 0 }; */
 
-GType
-gst_pipeline_get_type (void)
-{
-  static GType pipeline_type = 0;
-
-  if (G_UNLIKELY (pipeline_type == 0)) {
-    static const GTypeInfo pipeline_info = {
-      sizeof (GstPipelineClass),
-      gst_pipeline_base_init,
-      NULL,
-      (GClassInitFunc) gst_pipeline_class_init,
-      NULL,
-      NULL,
-      sizeof (GstPipeline),
-      0,
-      gst_pipeline_init,
-      NULL
-    };
-
-    pipeline_type =
-        g_type_register_static (GST_TYPE_BIN, "GstPipeline", &pipeline_info, 0);
-
-    GST_DEBUG_CATEGORY_INIT (pipeline_debug, "pipeline", GST_DEBUG_BOLD,
-        "debugging info for the 'pipeline' container element");
-  }
-  return pipeline_type;
+#define _do_init(type) \
+{ \
+  GST_DEBUG_CATEGORY_INIT (pipeline_debug, "pipeline", GST_DEBUG_BOLD, \
+      "debugging info for the 'pipeline' container element"); \
 }
 
+GST_BOILERPLATE_FULL (GstPipeline, gst_pipeline, GstBin, GST_TYPE_BIN,
+    _do_init);
+
 static void
 gst_pipeline_base_init (gpointer g_class)
 {
   GstElementClass *gstelement_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_set_details (gstelement_class, &gst_pipeline_details);
+  gst_element_class_set_details_simple (gstelement_class, "Pipeline object",
+      "Generic/Bin",
+      "Complete pipeline object",
+      "Erik Walthinsen <omega@cse.ogi.edu>, Wim Taymans <wim@fluendo.com>");
 }
 
 static void
-gst_pipeline_class_init (gpointer g_class, gpointer class_data)
+gst_pipeline_class_init (GstPipelineClass * klass)
 {
-  GObjectClass *gobject_class = G_OBJECT_CLASS (g_class);
-  GstElementClass *gstelement_class = GST_ELEMENT_CLASS (g_class);
-  GstPipelineClass *klass = GST_PIPELINE_CLASS (g_class);
-
-  parent_class = g_type_class_peek_parent (klass);
+  GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
+  GstElementClass *gstelement_class = GST_ELEMENT_CLASS (klass);
+  GstBinClass *gstbin_class = GST_BIN_CLASS (klass);
 
   g_type_class_add_private (klass, sizeof (GstPipelinePrivate));
 
-  gobject_class->set_property = GST_DEBUG_FUNCPTR (gst_pipeline_set_property);
-  gobject_class->get_property = GST_DEBUG_FUNCPTR (gst_pipeline_get_property);
+  gobject_class->set_property = gst_pipeline_set_property;
+  gobject_class->get_property = gst_pipeline_get_property;
 
   /**
    * GstPipeline:delay
@@ -207,7 +186,7 @@ gst_pipeline_class_init (gpointer g_class, gpointer class_data)
       g_param_spec_uint64 ("delay", "Delay",
           "Expected delay needed for elements "
           "to spin up to PLAYING in nanoseconds", 0, G_MAXUINT64, DEFAULT_DELAY,
-          G_PARAM_READWRITE));
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   /**
    * GstPipeline:auto-flush-bus:
@@ -222,21 +201,21 @@ gst_pipeline_class_init (gpointer g_class, gpointer class_data)
       g_param_spec_boolean ("auto-flush-bus", "Auto Flush Bus",
           "Whether to automatically flush the pipeline's bus when going "
           "from READY into NULL state", DEFAULT_AUTO_FLUSH_BUS,
-          G_PARAM_READWRITE));
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-  gobject_class->dispose = GST_DEBUG_FUNCPTR (gst_pipeline_dispose);
+  gobject_class->dispose = gst_pipeline_dispose;
 
-  gstelement_class->send_event = GST_DEBUG_FUNCPTR (gst_pipeline_send_event);
   gstelement_class->change_state =
       GST_DEBUG_FUNCPTR (gst_pipeline_change_state);
   gstelement_class->provide_clock =
       GST_DEBUG_FUNCPTR (gst_pipeline_provide_clock_func);
+  gstbin_class->handle_message =
+      GST_DEBUG_FUNCPTR (gst_pipeline_handle_message);
 }
 
 static void
-gst_pipeline_init (GTypeInstance * instance, gpointer g_class)
+gst_pipeline_init (GstPipeline * pipeline, GstPipelineClass * klass)
 {
-  GstPipeline *pipeline = GST_PIPELINE (instance);
   GstBus *bus;
 
   pipeline->priv = GST_PIPELINE_GET_PRIVATE (pipeline);
@@ -311,90 +290,20 @@ gst_pipeline_get_property (GObject * object, guint prop_id,
   }
 }
 
-/* default pipeline seeking code:
- *
- * If the pipeline is PLAYING and a flushing seek is done, set
- * the pipeline to PAUSED before doing the seek.
- *
- * A flushing seek also resets the stream time to 0 so that when
- * we go back to PLAYING after the seek, the base_time is recalculated
- * and redistributed to the elements.
- */
-static gboolean
-do_pipeline_seek (GstElement * element, GstEvent * event)
+/* set the start_time to 0, this will cause us to select a new base_time and
+ * make the running_time start from 0 again. */
+static void
+reset_start_time (GstPipeline * pipeline)
 {
-  gdouble rate;
-  GstSeekFlags flags;
-  gboolean flush;
-  gboolean was_playing = FALSE;
-  gboolean res;
-
-  /* we are only interested in the FLUSH flag of the seek event. */
-  gst_event_parse_seek (event, &rate, NULL, &flags, NULL, NULL, NULL, NULL);
-
-  flush = flags & GST_SEEK_FLAG_FLUSH;
-
-  /* if flushing seek, get the current state */
-  if (flush) {
-    GstState state;
-
-    /* need to call _get_state() since a bin state is only updated
-     * with this call. */
-    gst_element_get_state (element, &state, NULL, 0);
-    was_playing = state == GST_STATE_PLAYING;
-
-    if (was_playing) {
-      /* and PAUSE when the pipeline was PLAYING, we don't need
-       * to wait for the state change to complete since we are going
-       * to flush out any preroll sample anyway */
-      gst_element_set_state (element, GST_STATE_PAUSED);
-    }
+  GST_OBJECT_LOCK (pipeline);
+  if (GST_ELEMENT_START_TIME (pipeline) != GST_CLOCK_TIME_NONE) {
+    GST_DEBUG_OBJECT (pipeline, "reset start_time to 0");
+    GST_ELEMENT_START_TIME (pipeline) = 0;
+    pipeline->priv->last_start_time = -1;
+  } else {
+    GST_DEBUG_OBJECT (pipeline, "application asked to not reset stream_time");
   }
-
-  /* let parent class implement the seek behaviour */
-  res = GST_ELEMENT_CLASS (parent_class)->send_event (element, event);
-
-  /* if flushing seek restore previous state */
-  if (flush) {
-    gboolean need_reset;
-
-    GST_OBJECT_LOCK (element);
-    need_reset = GST_PIPELINE (element)->stream_time != GST_CLOCK_TIME_NONE;
-    GST_OBJECT_UNLOCK (element);
-
-    /* need to reset the stream time to 0 after a successfull flushing seek, 
-     * unless the user explicitly disabled this behavior by setting stream 
-     * time to NONE */
-    if (need_reset && res)
-      gst_pipeline_set_new_stream_time (GST_PIPELINE (element), 0);
-
-    if (was_playing)
-      /* and continue playing, this might return ASYNC in which case the
-       * application can wait for the PREROLL to complete after the seek. 
-       */
-      gst_element_set_state (element, GST_STATE_PLAYING);
-  }
-  return res;
-}
-
-static gboolean
-gst_pipeline_send_event (GstElement * element, GstEvent * event)
-{
-  gboolean res;
-  GstEventType event_type = GST_EVENT_TYPE (event);
-
-  switch (event_type) {
-    case GST_EVENT_SEEK:
-      /* do the default seek handling */
-      res = do_pipeline_seek (element, event);
-      break;
-    default:
-      /* else parent implements the defaults */
-      res = GST_ELEMENT_CLASS (parent_class)->send_event (element, event);
-      break;
-  }
-
-  return res;
+  GST_OBJECT_UNLOCK (pipeline);
 }
 
 /**
@@ -413,12 +322,49 @@ gst_pipeline_new (const gchar * name)
   return gst_element_factory_make ("pipeline", name);
 }
 
+/* takes a snapshot of the running_time of the pipeline and store this as the
+ * element start_time. This is the time we will set as the running_time of the
+ * pipeline when we go to PLAYING next. */
+static void
+pipeline_update_start_time (GstElement * element)
+{
+  GstPipeline *pipeline = GST_PIPELINE_CAST (element);
+  GstClock *clock;
+
+  GST_OBJECT_LOCK (element);
+  if ((clock = element->clock)) {
+    GstClockTime now;
+
+    gst_object_ref (clock);
+    GST_OBJECT_UNLOCK (element);
+
+    /* calculate the time when we stopped */
+    now = gst_clock_get_time (clock);
+    gst_object_unref (clock);
+
+    GST_OBJECT_LOCK (element);
+    /* store the current running time */
+    if (GST_ELEMENT_START_TIME (pipeline) != GST_CLOCK_TIME_NONE) {
+      GST_ELEMENT_START_TIME (pipeline) = now - element->base_time;
+      /* we went to PAUSED, when going to PLAYING select clock and new
+       * base_time */
+      pipeline->priv->update_clock = TRUE;
+    }
+    GST_DEBUG_OBJECT (element,
+        "start_time=%" GST_TIME_FORMAT ", now=%" GST_TIME_FORMAT
+        ", base_time %" GST_TIME_FORMAT,
+        GST_TIME_ARGS (GST_ELEMENT_START_TIME (pipeline)),
+        GST_TIME_ARGS (now), GST_TIME_ARGS (element->base_time));
+  }
+  GST_OBJECT_UNLOCK (element);
+}
+
 /* MT safe */
 static GstStateChangeReturn
 gst_pipeline_change_state (GstElement * element, GstStateChange transition)
 {
   GstStateChangeReturn result = GST_STATE_CHANGE_SUCCESS;
-  GstPipeline *pipeline = GST_PIPELINE (element);
+  GstPipeline *pipeline = GST_PIPELINE_CAST (element);
   GstClock *clock;
 
   switch (transition) {
@@ -429,59 +375,100 @@ gst_pipeline_change_state (GstElement * element, GstStateChange transition)
       GST_OBJECT_UNLOCK (element);
       break;
     case GST_STATE_CHANGE_READY_TO_PAUSED:
+      GST_OBJECT_LOCK (element);
+      pipeline->priv->update_clock = TRUE;
+      GST_OBJECT_UNLOCK (element);
       break;
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
     {
-      GstClockTime new_base_time;
+      GstClockTime now, start_time, last_start_time, delay;
+      gboolean update_clock;
+      GstClock *cur_clock;
 
-      /* when going to playing, select a clock */
-      clock = gst_element_provide_clock (element);
+      GST_DEBUG_OBJECT (element, "selecting clock and base_time");
 
-      if (clock) {
-        GstClockTime start_time, stream_time, delay;
-        gboolean new_clock;
+      GST_OBJECT_LOCK (element);
+      cur_clock = element->clock;
+      if (cur_clock)
+        gst_object_ref (cur_clock);
+      /* get the desired running_time of the first buffer aka the start_time */
+      start_time = GST_ELEMENT_START_TIME (pipeline);
+      last_start_time = pipeline->priv->last_start_time;
+      pipeline->priv->last_start_time = start_time;
+      /* see if we need to update the clock */
+      update_clock = pipeline->priv->update_clock;
+      pipeline->priv->update_clock = FALSE;
+      delay = pipeline->delay;
+      GST_OBJECT_UNLOCK (element);
 
-        start_time = gst_clock_get_time (clock);
+      /* running time changed, either with a PAUSED or a flush, we need to check
+       * if there is a new clock & update the base time */
+      if (last_start_time != start_time) {
+        GST_DEBUG_OBJECT (pipeline, "Need to update start_time");
 
-        GST_OBJECT_LOCK (element);
-        new_clock = element->clock != clock;
-        stream_time = pipeline->stream_time;
-        delay = pipeline->delay;
-        GST_OBJECT_UNLOCK (element);
+        /* when going to PLAYING, select a clock when needed. If we just got
+         * flushed, we don't reselect the clock. */
+        if (update_clock) {
+          GST_DEBUG_OBJECT (pipeline, "Need to update clock.");
+          clock = gst_element_provide_clock (element);
+        } else {
+          GST_DEBUG_OBJECT (pipeline,
+              "Don't need to update clock, using old clock.");
+          clock = gst_object_ref (cur_clock);
+        }
 
-        if (new_clock) {
+        if (clock) {
+          now = gst_clock_get_time (clock);
+        } else {
+          GST_DEBUG ("no clock, using base time of NONE");
+          now = GST_CLOCK_TIME_NONE;
+        }
+
+        if (clock != cur_clock) {
           /* now distribute the clock (which could be NULL). If some
            * element refuses the clock, this will return FALSE and
            * we effectively fail the state change. */
           if (!gst_element_set_clock (element, clock))
             goto invalid_clock;
 
-          /* if we selected and distributed a new clock, let the app 
+          /* if we selected and distributed a new clock, let the app
            * know about it */
           gst_element_post_message (element,
               gst_message_new_new_clock (GST_OBJECT_CAST (element), clock));
         }
 
-        if (stream_time != GST_CLOCK_TIME_NONE)
-          new_base_time = start_time - stream_time + delay;
-        else
-          new_base_time = GST_CLOCK_TIME_NONE;
+        if (clock)
+          gst_object_unref (clock);
 
-        gst_object_unref (clock);
+        if (start_time != GST_CLOCK_TIME_NONE && now != GST_CLOCK_TIME_NONE) {
+          GstClockTime new_base_time = now - start_time + delay;
+          GST_DEBUG_OBJECT (element,
+              "start_time=%" GST_TIME_FORMAT ", now=%" GST_TIME_FORMAT
+              ", base_time %" GST_TIME_FORMAT,
+              GST_TIME_ARGS (start_time), GST_TIME_ARGS (now),
+              GST_TIME_ARGS (new_base_time));
+
+          gst_element_set_base_time (element, new_base_time);
+        } else {
+          GST_DEBUG_OBJECT (pipeline,
+              "NOT adjusting base_time because start_time is NONE");
+        }
       } else {
-        GST_DEBUG ("no clock, using base time of 0");
-        new_base_time = 0;
+        GST_DEBUG_OBJECT (pipeline,
+            "NOT adjusting base_time because we selected one before");
       }
 
-      if (new_base_time != GST_CLOCK_TIME_NONE)
-        gst_element_set_base_time (element, new_base_time);
-      else
-        GST_DEBUG_OBJECT (pipeline,
-            "NOT adjusting base time because stream time is NONE");
+      if (cur_clock)
+        gst_object_unref (cur_clock);
+      break;
     }
-      break;
     case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
+    {
+      /* we take a start_time snapshot before calling the children state changes
+       * so that they know about when the pipeline PAUSED. */
+      pipeline_update_start_time (element);
       break;
+    }
     case GST_STATE_CHANGE_PAUSED_TO_READY:
     case GST_STATE_CHANGE_READY_TO_NULL:
       break;
@@ -494,58 +481,43 @@ gst_pipeline_change_state (GstElement * element, GstStateChange transition)
       break;
     case GST_STATE_CHANGE_READY_TO_PAUSED:
     {
-      gboolean need_reset;
-
-      /* only reset the stream time when the application did not
-       * specify a stream time explicitly */
-      GST_OBJECT_LOCK (element);
-      need_reset = pipeline->stream_time != GST_CLOCK_TIME_NONE;
-      GST_OBJECT_UNLOCK (element);
-
-      if (need_reset)
-        gst_pipeline_set_new_stream_time (pipeline, 0);
-
+      reset_start_time (pipeline);
       break;
     }
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
       break;
     case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
-      GST_OBJECT_LOCK (element);
-      if ((clock = element->clock)) {
-        GstClockTime now;
-
-        gst_object_ref (clock);
-        GST_OBJECT_UNLOCK (element);
-
-        /* calculate the time when we stopped */
-        now = gst_clock_get_time (clock);
-        gst_object_unref (clock);
-
-        GST_OBJECT_LOCK (element);
-        /* store the current stream time */
-        if (pipeline->stream_time != GST_CLOCK_TIME_NONE)
-          pipeline->stream_time = now - element->base_time;
-        GST_DEBUG_OBJECT (element,
-            "stream_time=%" GST_TIME_FORMAT ", now=%" GST_TIME_FORMAT
-            ", base time %" GST_TIME_FORMAT,
-            GST_TIME_ARGS (pipeline->stream_time), GST_TIME_ARGS (now),
-            GST_TIME_ARGS (element->base_time));
-      }
-      GST_OBJECT_UNLOCK (element);
+    {
+      /* Take a new snapshot of the start_time after calling the state change on
+       * all children. This will be the running_time of the pipeline when we go
+       * back to PLAYING */
+      pipeline_update_start_time (element);
       break;
+    }
     case GST_STATE_CHANGE_PAUSED_TO_READY:
       break;
     case GST_STATE_CHANGE_READY_TO_NULL:
+    {
+      GstBus *bus;
+      gboolean auto_flush;
+
+      /* grab some stuff before we release the lock to flush out the bus */
       GST_OBJECT_LOCK (element);
-      if (element->bus) {
-        if (pipeline->priv->auto_flush_bus) {
-          gst_bus_set_flushing (element->bus, TRUE);
+      if ((bus = element->bus))
+        gst_object_ref (bus);
+      auto_flush = pipeline->priv->auto_flush_bus;
+      GST_OBJECT_UNLOCK (element);
+
+      if (bus) {
+        if (auto_flush) {
+          gst_bus_set_flushing (bus, TRUE);
         } else {
           GST_INFO_OBJECT (element, "not flushing bus, auto-flushing disabled");
         }
+        gst_object_unref (bus);
       }
-      GST_OBJECT_UNLOCK (element);
       break;
+    }
   }
   return result;
 
@@ -559,15 +531,49 @@ invalid_clock:
         ("Pipeline cannot operate with selected clock"));
     GST_DEBUG_OBJECT (pipeline,
         "Pipeline cannot operate with selected clock %p", clock);
+    if (clock)
+      gst_object_unref (clock);
     return GST_STATE_CHANGE_FAILURE;
   }
 }
 
+/* intercept the bus messages from our children. We watch for the ASYNC_START
+ * message with is posted by the elements (sinks) that require a reset of the
+ * running_time after a flush. ASYNC_START also brings the pipeline back into
+ * the PAUSED, pending PAUSED state. When the ASYNC_DONE message is received the
+ * pipeline will redistribute the new base_time and will bring the elements back
+ * to the desired state of the pipeline. */
+static void
+gst_pipeline_handle_message (GstBin * bin, GstMessage * message)
+{
+  GstPipeline *pipeline = GST_PIPELINE_CAST (bin);
+
+  switch (GST_MESSAGE_TYPE (message)) {
+    case GST_MESSAGE_ASYNC_START:
+    {
+      gboolean new_base_time;
+
+      gst_message_parse_async_start (message, &new_base_time);
+
+      /* reset our running time if we need to distribute a new base_time to the
+       * children. */
+      if (new_base_time)
+        reset_start_time (pipeline);
+
+      break;
+    }
+    default:
+      break;
+  }
+  GST_BIN_CLASS (parent_class)->handle_message (bin, message);
+}
+
 /**
  * gst_pipeline_get_bus:
  * @pipeline: a #GstPipeline
  *
- * Gets the #GstBus of @pipeline.
+ * Gets the #GstBus of @pipeline. The bus allows applications to receive
+ * #GstMessage packets.
  *
  * Returns: a #GstBus, unref after usage.
  *
@@ -576,15 +582,15 @@ invalid_clock:
 GstBus *
 gst_pipeline_get_bus (GstPipeline * pipeline)
 {
-  return gst_element_get_bus (GST_ELEMENT (pipeline));
+  return gst_element_get_bus (GST_ELEMENT_CAST (pipeline));
 }
 
 /**
  * gst_pipeline_set_new_stream_time:
  * @pipeline: a #GstPipeline
- * @time: the new stream time to set
+ * @time: the new running time to set
  *
- * Set the new stream time of @pipeline to @time. The stream time is used to
+ * Set the new start time of @pipeline to @time. The start time is used to
  * set the base time on the elements (see gst_element_set_base_time())
  * in the PAUSED->PLAYING state transition.
  *
@@ -595,40 +601,44 @@ gst_pipeline_get_bus (GstPipeline * pipeline)
  * pipelines have the same clock.
  *
  * MT safe.
+ *
+ * Deprecated: This function has the wrong name and is equivalent to
+ * gst_element_set_start_time(). 
  */
+#ifndef GST_REMOVE_DEPRECATED
 void
 gst_pipeline_set_new_stream_time (GstPipeline * pipeline, GstClockTime time)
 {
   g_return_if_fail (GST_IS_PIPELINE (pipeline));
 
-  GST_OBJECT_LOCK (pipeline);
-  pipeline->stream_time = time;
-  GST_OBJECT_UNLOCK (pipeline);
-
-  GST_DEBUG_OBJECT (pipeline, "set new stream_time to %" GST_TIME_FORMAT,
-      GST_TIME_ARGS (time));
+  gst_element_set_start_time (GST_ELEMENT_CAST (pipeline), time);
 
   if (time == GST_CLOCK_TIME_NONE)
-    GST_DEBUG_OBJECT (pipeline, "told not to adjust base time");
+    GST_DEBUG_OBJECT (pipeline, "told not to adjust base_time");
 }
+#endif /* GST_REMOVE_DEPRECATED */
 
 /**
  * gst_pipeline_get_last_stream_time:
  * @pipeline: a #GstPipeline
  *
- * Gets the last stream time of @pipeline. If the pipeline is PLAYING,
- * the returned time is the stream time used to configure the element's
- * base time in the PAUSED->PLAYING state. If the pipeline is PAUSED, the 
- * returned time is the stream time when the pipeline was paused.
+ * Gets the last running time of @pipeline. If the pipeline is PLAYING,
+ * the returned time is the running time used to configure the element's
+ * base time in the PAUSED->PLAYING state. If the pipeline is PAUSED, the
+ * returned time is the running time when the pipeline was paused.
  *
  * This function returns #GST_CLOCK_TIME_NONE if the pipeline was
- * configured to not handle the management of the element's base time 
+ * configured to not handle the management of the element's base time
  * (see gst_pipeline_set_new_stream_time()).
  *
+ * MT safe.
+ *
  * Returns: a #GstClockTime.
  *
- * MT safe.
+ * Deprecated: This function has the wrong name and is equivalent to
+ * gst_element_get_start_time(). 
  */
+#ifndef GST_REMOVE_DEPRECATED
 GstClockTime
 gst_pipeline_get_last_stream_time (GstPipeline * pipeline)
 {
@@ -636,12 +646,11 @@ gst_pipeline_get_last_stream_time (GstPipeline * pipeline)
 
   g_return_val_if_fail (GST_IS_PIPELINE (pipeline), GST_CLOCK_TIME_NONE);
 
-  GST_OBJECT_LOCK (pipeline);
-  result = pipeline->stream_time;
-  GST_OBJECT_UNLOCK (pipeline);
+  result = gst_element_get_start_time (GST_ELEMENT_CAST (pipeline));
 
   return result;
 }
+#endif /* GST_REMOVE_DEPRECATED */
 
 static GstClock *
 gst_pipeline_provide_clock_func (GstElement * element)
@@ -653,7 +662,8 @@ gst_pipeline_provide_clock_func (GstElement * element)
   GST_OBJECT_LOCK (pipeline);
   if (GST_OBJECT_FLAG_IS_SET (pipeline, GST_PIPELINE_FLAG_FIXED_CLOCK)) {
     clock = pipeline->fixed_clock;
-    gst_object_ref (clock);
+    if (clock)
+      gst_object_ref (clock);
     GST_OBJECT_UNLOCK (pipeline);
 
     GST_CAT_DEBUG (GST_CAT_CLOCK, "pipeline using fixed clock %p (%s)",
@@ -662,8 +672,8 @@ gst_pipeline_provide_clock_func (GstElement * element)
     GST_OBJECT_UNLOCK (pipeline);
     /* let the parent bin select a clock */
     clock =
-        GST_ELEMENT_CLASS (parent_class)->
-        provide_clock (GST_ELEMENT (pipeline));
+        GST_ELEMENT_CLASS (parent_class)->provide_clock (GST_ELEMENT
+        (pipeline));
     /* no clock, use a system clock */
     if (!clock) {
       clock = gst_system_clock_obtain ();
@@ -691,7 +701,7 @@ gst_pipeline_get_clock (GstPipeline * pipeline)
 {
   g_return_val_if_fail (GST_IS_PIPELINE (pipeline), NULL);
 
-  return gst_pipeline_provide_clock_func (GST_ELEMENT (pipeline));
+  return gst_pipeline_provide_clock_func (GST_ELEMENT_CAST (pipeline));
 }
 
 
@@ -733,7 +743,7 @@ gst_pipeline_use_clock (GstPipeline * pipeline, GstClock * clock)
  * @clock: the clock to set
  *
  * Set the clock for @pipeline. The clock will be distributed
- * to all the elements managed by the pipeline. 
+ * to all the elements managed by the pipeline.
  *
  * Returns: TRUE if the clock could be set on the pipeline. FALSE if
  *   some element did not accept the clock.
@@ -746,7 +756,8 @@ gst_pipeline_set_clock (GstPipeline * pipeline, GstClock * clock)
   g_return_val_if_fail (pipeline != NULL, FALSE);
   g_return_val_if_fail (GST_IS_PIPELINE (pipeline), FALSE);
 
-  return GST_ELEMENT_CLASS (parent_class)->set_clock (GST_ELEMENT (pipeline),
+  return
+      GST_ELEMENT_CLASS (parent_class)->set_clock (GST_ELEMENT_CAST (pipeline),
       clock);
 }
 
@@ -755,9 +766,9 @@ gst_pipeline_set_clock (GstPipeline * pipeline, GstClock * clock)
  * @pipeline: a #GstPipeline
  *
  * Let @pipeline select a clock automatically. This is the default
- * behaviour. 
+ * behaviour.
  *
- * Use this function if you previous forced a fixed clock with 
+ * Use this function if you previous forced a fixed clock with
  * gst_pipeline_use_clock() and want to restore the default
  * pipeline clock selection algorithm.
  *
@@ -789,9 +800,10 @@ gst_pipeline_auto_clock (GstPipeline * pipeline)
  * Set the expected delay needed for all elements to perform the
  * PAUSED to PLAYING state change. @delay will be added to the
  * base time of the elements so that they wait an additional @delay
- * amount of time before starting to process buffers.
+ * amount of time before starting to process buffers and cannot be
+ * #GST_CLOCK_TIME_NONE.
  *
- * This option is used for tuning purposes and should normally not be 
+ * This option is used for tuning purposes and should normally not be
  * used.
  *
  * MT safe.
@@ -802,6 +814,7 @@ void
 gst_pipeline_set_delay (GstPipeline * pipeline, GstClockTime delay)
 {
   g_return_if_fail (GST_IS_PIPELINE (pipeline));
+  g_return_if_fail (delay != GST_CLOCK_TIME_NONE);
 
   GST_OBJECT_LOCK (pipeline);
   pipeline->delay = delay;
@@ -842,15 +855,15 @@ gst_pipeline_get_delay (GstPipeline * pipeline)
  *
  * Usually, when a pipeline goes from READY to NULL state, it automatically
  * flushes all pending messages on the bus, which is done for refcounting
- * purposes, to break circular references. 
+ * purposes, to break circular references.
  *
- * This means that applications that update state using (async) bus messages 
- * (e.g. do certain things when a pipeline goes from PAUSED to READY) might 
- * not get to see messages when the pipeline is shut down, because they might 
+ * This means that applications that update state using (async) bus messages
+ * (e.g. do certain things when a pipeline goes from PAUSED to READY) might
+ * not get to see messages when the pipeline is shut down, because they might
  * be flushed before they can be dispatched in the main thread. This behaviour
  * can be disabled using this function.
  *
- * It is important that all messages on the bus are handled when the 
+ * It is important that all messages on the bus are handled when the
  * automatic flushing is disabled else memory leaks will be introduced.
  *
  * MT safe.
