@@ -110,6 +110,7 @@ gst_object_uncontrol_properties (GObject * object, ...)
  * Returns: the controller handling some of the given element's properties, %NULL if no controller
  * Since: 0.9
  */
+/* FIXME 0.11: This should return a new reference */
 GstController *
 gst_object_get_controller (GObject * object)
 {
@@ -128,15 +129,14 @@ gst_object_get_controller (GObject * object)
  * Returns: %FALSE if the GObject already has an controller, %TRUE otherwise
  * Since: 0.9
  */
+/* FIXME 0.11: This should increase the refcount before storing */
 gboolean
 gst_object_set_controller (GObject * object, GstController * controller)
 {
-  GstController *ctrl;
-
   g_return_val_if_fail (G_IS_OBJECT (object), FALSE);
   g_return_val_if_fail (controller, FALSE);
 
-  if (!(ctrl = g_object_get_qdata (object, priv_gst_controller_key))) {
+  if (!g_object_get_qdata (object, priv_gst_controller_key)) {
     g_object_set_qdata (object, priv_gst_controller_key, controller);
     return (TRUE);
   }
@@ -181,7 +181,6 @@ gst_object_sync_values (GObject * object, GstClockTime timestamp)
   GstController *ctrl = NULL;
 
   g_return_val_if_fail (G_IS_OBJECT (object), FALSE);
-  g_return_val_if_fail (GST_CLOCK_TIME_IS_VALID (timestamp), FALSE);
 
   if ((ctrl = g_object_get_qdata (object, priv_gst_controller_key))) {
     return gst_controller_sync_values (ctrl, timestamp);
