@@ -60,8 +60,9 @@
 #  include "config.h"
 #endif
 
-#include "gsttypefindelement.h"
 #include "gst/gst_private.h"
+
+#include "gsttypefindelement.h"
 #include "gst/gst-i18n-lib.h"
 #include "gst/base/gsttypefindhelper.h"
 
@@ -160,14 +161,22 @@ static void
 gst_type_find_element_have_type (GstTypeFindElement * typefind,
     guint probability, const GstCaps * caps)
 {
+  GstCaps *copy;
+
   g_assert (caps != NULL);
 
   GST_INFO_OBJECT (typefind, "found caps %" GST_PTR_FORMAT ", probability=%u",
       caps, probability);
+
+  GST_OBJECT_LOCK (typefind);
   if (typefind->caps)
     gst_caps_unref (typefind->caps);
   typefind->caps = gst_caps_copy (caps);
-  gst_pad_set_caps (typefind->src, (GstCaps *) caps);
+  copy = gst_caps_ref (typefind->caps);
+  GST_OBJECT_UNLOCK (typefind);
+
+  gst_pad_set_caps (typefind->src, copy);
+  gst_caps_unref (copy);
 }
 
 static void
@@ -192,13 +201,9 @@ gst_type_find_element_class_init (GstTypeFindElementClass * typefind_class)
   GObjectClass *gobject_class = G_OBJECT_CLASS (typefind_class);
   GstElementClass *gstelement_class = GST_ELEMENT_CLASS (typefind_class);
 
-  gobject_class->set_property =
-      GST_DEBUG_FUNCPTR (gst_type_find_element_set_property);
-  gobject_class->get_property =
-      GST_DEBUG_FUNCPTR (gst_type_find_element_get_property);
-  gobject_class->dispose = GST_DEBUG_FUNCPTR (gst_type_find_element_dispose);
-
-  typefind_class->have_type = gst_type_find_element_have_type;
+  gobject_class->set_property = gst_type_find_element_set_property;
+  gobject_class->get_property = gst_type_find_element_get_property;
+  gobject_class->dispose = gst_type_find_element_dispose;
 
   g_object_class_install_property (gobject_class, PROP_CAPS,
       g_param_spec_boxed ("caps", _("caps"),
@@ -233,6 +238,9 @@ gst_type_find_element_class_init (GstTypeFindElementClass * typefind_class)
       gst_marshal_VOID__UINT_BOXED, G_TYPE_NONE, 2,
       G_TYPE_UINT, GST_TYPE_CAPS | G_SIGNAL_TYPE_STATIC_SCOPE);
 
+  typefind_class->have_type =
+      GST_DEBUG_FUNCPTR (gst_type_find_element_have_type);
+
   gstelement_class->change_state =
       GST_DEBUG_FUNCPTR (gst_type_find_element_change_state);
 }
@@ -290,6 +298,7 @@ gst_type_find_element_dispose (GObject * object)
     gst_buffer_unref (typefind->store);
     typefind->store = NULL;
   }
+
   if (typefind->force_caps) {
     gst_caps_unref (typefind->force_caps);
     typefind->force_caps = NULL;
@@ -336,7 +345,9 @@ gst_type_find_element_get_property (GObject * object, guint prop_id,
 
   switch (prop_id) {
     case PROP_CAPS:
+      GST_OBJECT_LOCK (typefind);
       g_value_set_boxed (value, typefind->caps);
+      GST_OBJECT_UNLOCK (typefind);
       break;
     case PROP_MINIMUM:
       g_value_set_uint (value, typefind->min_probability);
@@ -378,8 +389,11 @@ gst_type_find_handle_src_query (GstPad * pad, GstQuery * query)
       gint64 peer_pos;
       GstFormat format;
 
-      if (typefind->store == NULL)
+      GST_OBJECT_LOCK (typefind);
+      if (typefind->store == NULL) {
+        GST_OBJECT_UNLOCK (typefind);
         goto out;
+      }
 
       gst_query_parse_position (query, &format, &peer_pos);
 
@@ -392,6 +406,7 @@ gst_type_find_handle_src_query (GstPad * pad, GstQuery * query)
           /* FIXME */
           break;
       }
+      GST_OBJECT_UNLOCK (typefind);
       gst_query_set_position (query, format, peer_pos);
       break;
     }
@@ -438,9 +453,12 @@ start_typefinding (GstTypeFindElement * typefind)
 {
   GST_DEBUG_OBJECT (typefind, "starting typefinding");
   gst_pad_set_caps (typefind->src, NULL);
-  if (typefind->caps) {
+
+  GST_OBJECT_LOCK (typefind);
+  if (typefind->caps)
     gst_caps_replace (&typefind->caps, NULL);
-  }
+  GST_OBJECT_UNLOCK (typefind);
+
   typefind->mode = MODE_TYPEFIND;
 }
 
@@ -457,14 +475,20 @@ stop_typefinding (GstTypeFindElement * typefind)
   GST_DEBUG_OBJECT (typefind, "stopping typefinding%s",
       push_cached_buffers ? " and pushing cached buffers" : "");
 
+  GST_OBJECT_LOCK (typefind);
   if (typefind->store) {
+    GstBuffer *store = typefind->store;
+
+    typefind->store = NULL;
+    gst_buffer_set_caps (store, typefind->caps);
+    GST_OBJECT_UNLOCK (typefind);
+
     if (!push_cached_buffers) {
-      gst_buffer_unref (typefind->store);
+      gst_buffer_unref (store);
     } else {
       GstPad *peer = gst_pad_get_peer (typefind->src);
 
       typefind->mode = MODE_NORMAL;
-      gst_buffer_set_caps (typefind->store, typefind->caps);
 
       /* make sure the user gets a meaningful error message in this case,
        * which is not a core bug or bug of any kind (as the default error
@@ -480,15 +504,17 @@ stop_typefinding (GstTypeFindElement * typefind)
                 "element does not support pull mode",
                 GST_DEBUG_PAD_NAME (peer)));
         typefind->mode = MODE_ERROR;    /* make the chain function error out */
+        gst_buffer_unref (store);
       } else {
         gst_type_find_element_send_cached_events (typefind);
-        gst_pad_push (typefind->src, typefind->store);
+        gst_pad_push (typefind->src, store);
       }
 
       if (peer)
         gst_object_unref (peer);
     }
-    typefind->store = NULL;
+  } else {
+    GST_OBJECT_UNLOCK (typefind);
   }
 }
 
@@ -512,9 +538,11 @@ gst_type_find_element_handle_event (GstPad * pad, GstEvent * event)
 
           /* we might not have started typefinding yet because there was not
            * enough data so far; just give it a shot now and see what we get */
+          GST_OBJECT_LOCK (typefind);
           if (typefind->store) {
             caps = gst_type_find_helper_for_buffer (GST_OBJECT (typefind),
                 typefind->store, &prob);
+            GST_OBJECT_UNLOCK (typefind);
 
             if (caps && prob >= typefind->min_probability) {
               g_signal_emit (typefind, gst_type_find_element_signals[HAVE_TYPE],
@@ -525,6 +553,7 @@ gst_type_find_element_handle_event (GstPad * pad, GstEvent * event)
             }
             gst_caps_replace (&caps, NULL);
           } else {
+            GST_OBJECT_UNLOCK (typefind);
             /* keep message in sync with the one in the pad activate function */
             GST_ELEMENT_ERROR (typefind, STREAM, TYPE_NOT_FOUND,
                 (_("Stream contains no data.")),
@@ -536,11 +565,13 @@ gst_type_find_element_handle_event (GstPad * pad, GstEvent * event)
           break;
         }
         case GST_EVENT_FLUSH_STOP:
+          GST_OBJECT_LOCK (typefind);
           g_list_foreach (typefind->cached_events,
               (GFunc) gst_mini_object_unref, NULL);
           g_list_free (typefind->cached_events);
           typefind->cached_events = NULL;
           gst_buffer_replace (&typefind->store, NULL);
+          GST_OBJECT_UNLOCK (typefind);
           /* fall through */
         case GST_EVENT_FLUSH_START:
           res = gst_pad_push_event (typefind->src, event);
@@ -548,8 +579,10 @@ gst_type_find_element_handle_event (GstPad * pad, GstEvent * event)
         default:
           GST_DEBUG_OBJECT (typefind, "Saving %s event to send later",
               GST_EVENT_TYPE_NAME (event));
+          GST_OBJECT_LOCK (typefind);
           typefind->cached_events =
               g_list_append (typefind->cached_events, event);
+          GST_OBJECT_UNLOCK (typefind);
           res = TRUE;
           break;
       }
@@ -568,17 +601,21 @@ gst_type_find_element_handle_event (GstPad * pad, GstEvent * event)
 static void
 gst_type_find_element_send_cached_events (GstTypeFindElement * typefind)
 {
-  GList *l;
+  GList *l, *cached_events;
 
-  for (l = typefind->cached_events; l != NULL; l = l->next) {
+  GST_OBJECT_LOCK (typefind);
+  cached_events = typefind->cached_events;
+  typefind->cached_events = NULL;
+  GST_OBJECT_UNLOCK (typefind);
+
+  for (l = cached_events; l != NULL; l = l->next) {
     GstEvent *event = GST_EVENT (l->data);
 
     GST_DEBUG_OBJECT (typefind, "sending cached %s event",
         GST_EVENT_TYPE_NAME (event));
     gst_pad_push_event (typefind->src, event);
   }
-  g_list_free (typefind->cached_events);
-  typefind->cached_events = NULL;
+  g_list_free (cached_events);
 }
 
 static gboolean
@@ -602,27 +639,32 @@ gst_type_find_element_setcaps (GstPad * pad, GstCaps * caps)
     typefind->mode = MODE_NORMAL;
 
     gst_type_find_element_send_cached_events (typefind);
+    GST_OBJECT_LOCK (typefind);
     if (typefind->store) {
-      GST_DEBUG_OBJECT (typefind, "Pushing store: %d",
-          GST_BUFFER_SIZE (typefind->store));
-      gst_buffer_set_caps (typefind->store, typefind->caps);
-      gst_pad_push (typefind->src, typefind->store);
+      GstBuffer *store = typefind->store;
+
       typefind->store = NULL;
+      GST_DEBUG_OBJECT (typefind, "Pushing store: %d", GST_BUFFER_SIZE (store));
+
+      gst_buffer_set_caps (store, typefind->caps);
+      GST_OBJECT_UNLOCK (typefind);
+
+      gst_pad_push (typefind->src, store);
+    } else {
+      GST_OBJECT_UNLOCK (typefind);
     }
   }
 
   return TRUE;
 }
 
-static GstCaps *
-gst_type_find_guess_by_extension (GstTypeFindElement * typefind, GstPad * pad,
-    GstTypeFindProbability * probability)
+static gchar *
+gst_type_find_get_extension (GstTypeFindElement * typefind, GstPad * pad)
 {
   GstQuery *query;
-  gchar *uri;
+  gchar *uri, *result;
   size_t len;
   gint find;
-  GstCaps *caps;
 
   query = gst_query_new_uri ();
 
@@ -648,17 +690,13 @@ gst_type_find_guess_by_extension (GstTypeFindElement * typefind, GstPad * pad,
   if (find < 0)
     goto no_extension;
 
-  GST_DEBUG_OBJECT (typefind, "found extension %s", &uri[find + 1]);
-
-  caps =
-      gst_type_find_helper_for_extension (GST_OBJECT_CAST (typefind),
-      &uri[find + 1]);
-  if (caps)
-    *probability = GST_TYPE_FIND_MAXIMUM;
+  result = g_strdup (&uri[find + 1]);
 
+  GST_DEBUG_OBJECT (typefind, "found extension %s", result);
   gst_query_unref (query);
+  g_free (uri);
 
-  return caps;
+  return result;
 
   /* ERRORS */
 peer_query_failed:
@@ -677,10 +715,31 @@ no_extension:
   {
     GST_WARNING_OBJECT (typefind, "could not find uri extension in %s", uri);
     gst_query_unref (query);
+    g_free (uri);
     return NULL;
   }
 }
 
+static GstCaps *
+gst_type_find_guess_by_extension (GstTypeFindElement * typefind, GstPad * pad,
+    GstTypeFindProbability * probability)
+{
+  gchar *ext;
+  GstCaps *caps;
+
+  ext = gst_type_find_get_extension (typefind, pad);
+  if (!ext)
+    return NULL;
+
+  caps = gst_type_find_helper_for_extension (GST_OBJECT_CAST (typefind), ext);
+  if (caps)
+    *probability = GST_TYPE_FIND_MAXIMUM;
+
+  g_free (ext);
+
+  return caps;
+}
+
 static GstFlowReturn
 gst_type_find_element_chain (GstPad * pad, GstBuffer * buffer)
 {
@@ -694,13 +753,17 @@ gst_type_find_element_chain (GstPad * pad, GstBuffer * buffer)
       /* we should already have called GST_ELEMENT_ERROR */
       return GST_FLOW_ERROR;
     case MODE_NORMAL:
+      GST_OBJECT_LOCK (typefind);
       gst_buffer_set_caps (buffer, typefind->caps);
+      GST_OBJECT_UNLOCK (typefind);
       return gst_pad_push (typefind->src, buffer);
     case MODE_TYPEFIND:{
+      GST_OBJECT_LOCK (typefind);
       if (typefind->store)
         typefind->store = gst_buffer_join (typefind->store, buffer);
       else
         typefind->store = buffer;
+      GST_OBJECT_UNLOCK (typefind);
 
       res = gst_type_find_element_chain_do_typefinding (typefind);
 
@@ -723,20 +786,23 @@ gst_type_find_element_chain_do_typefinding (GstTypeFindElement * typefind)
   GstTypeFindProbability probability;
   GstCaps *caps;
 
+  GST_OBJECT_LOCK (typefind);
   if (GST_BUFFER_SIZE (typefind->store) < TYPE_FIND_MIN_SIZE) {
     GST_DEBUG_OBJECT (typefind, "not enough data for typefinding yet "
         "(%u bytes)", GST_BUFFER_SIZE (typefind->store));
+    GST_OBJECT_UNLOCK (typefind);
     return GST_FLOW_OK;
   }
 
   caps = gst_type_find_helper_for_buffer (GST_OBJECT (typefind),
       typefind->store, &probability);
-
   if (caps == NULL && GST_BUFFER_SIZE (typefind->store) > TYPE_FIND_MAX_SIZE) {
+    GST_OBJECT_UNLOCK (typefind);
     GST_ELEMENT_ERROR (typefind, STREAM, TYPE_NOT_FOUND, (NULL), (NULL));
     stop_typefinding (typefind);
     return GST_FLOW_ERROR;
   } else if (caps == NULL) {
+    GST_OBJECT_UNLOCK (typefind);
     GST_DEBUG_OBJECT (typefind, "no caps found with %u bytes of data, "
         "waiting for more data", GST_BUFFER_SIZE (typefind->store));
     return GST_FLOW_OK;
@@ -751,14 +817,17 @@ gst_type_find_element_chain_do_typefinding (GstTypeFindElement * typefind)
     gst_caps_replace (&caps, NULL);
 
     if (GST_BUFFER_SIZE (typefind->store) >= TYPE_FIND_MAX_SIZE) {
+      GST_OBJECT_UNLOCK (typefind);
       GST_ELEMENT_ERROR (typefind, STREAM, TYPE_NOT_FOUND, (NULL), (NULL));
       stop_typefinding (typefind);
       return GST_FLOW_ERROR;
     }
 
+    GST_OBJECT_UNLOCK (typefind);
     GST_DEBUG_OBJECT (typefind, "waiting for more data to try again");
     return GST_FLOW_OK;
   }
+  GST_OBJECT_UNLOCK (typefind);
 
   /* probability is good enough too, so let's make it known ... */
   g_signal_emit (typefind, gst_type_find_element_signals[HAVE_TYPE], 0,
@@ -791,8 +860,11 @@ gst_type_find_element_getrange (GstPad * srcpad,
 
   ret = gst_pad_pull_range (typefind->sink, offset, length, buffer);
 
-  if (ret == GST_FLOW_OK && buffer && *buffer)
+  if (ret == GST_FLOW_OK && buffer && *buffer) {
+    GST_OBJECT_LOCK (typefind);
     gst_buffer_set_caps (*buffer, typefind->caps);
+    GST_OBJECT_UNLOCK (typefind);
+  }
 
   return ret;
 }
@@ -817,11 +889,14 @@ gst_type_find_element_activate (GstPad * pad)
   typefind = GST_TYPE_FIND_ELEMENT (GST_OBJECT_PARENT (pad));
 
   /* if we have force caps, use those */
+  GST_OBJECT_LOCK (typefind);
   if (typefind->force_caps) {
     found_caps = gst_caps_ref (typefind->force_caps);
     probability = GST_TYPE_FIND_MAXIMUM;
+    GST_OBJECT_UNLOCK (typefind);
     goto done;
   }
+  GST_OBJECT_UNLOCK (typefind);
 
   /* 1. try to activate in pull mode. if not, switch to push and succeed.
      2. try to pull type find.
@@ -849,6 +924,7 @@ gst_type_find_element_activate (GstPad * pad)
     if (peer) {
       gint64 size;
       GstFormat format = GST_FORMAT_BYTES;
+      gchar *ext;
 
       if (!gst_pad_query_duration (peer, &format, &size)) {
         GST_WARNING_OBJECT (typefind, "Could not query upstream length!");
@@ -864,15 +940,29 @@ gst_type_find_element_activate (GstPad * pad)
         gst_object_unref (peer);
         return FALSE;
       }
+      ext = gst_type_find_get_extension (typefind, pad);
 
-      found_caps = gst_type_find_helper_get_range (GST_OBJECT_CAST (peer),
+      found_caps = gst_type_find_helper_get_range_ext (GST_OBJECT_CAST (peer),
           (GstTypeFindHelperGetRangeFunction) (GST_PAD_GETRANGEFUNC (peer)),
-          (guint64) size, &probability);
+          (guint64) size, ext, &probability);
+      g_free (ext);
 
       gst_object_unref (peer);
     }
   }
 
+  /* the type find helpers might have triggered setcaps here (due to upstream)
+   * setting caps on buffers, which emits typefound signal and an element
+   * could have been linked and have its pads activated
+   *
+   * If we deactivate the pads in the following steps we might mess up
+   * downstream element. We should prevent that.
+   */
+  if (typefind->mode == MODE_NORMAL) {
+    /* this means we already emitted typefound */
+    goto really_done;
+  }
+
   /* 3 */
   gst_pad_activate_pull (pad, FALSE);
 
@@ -895,8 +985,9 @@ done:
   /* 7 */
   g_signal_emit (typefind, gst_type_find_element_signals[HAVE_TYPE],
       0, probability, found_caps);
-  gst_caps_unref (found_caps);
   typefind->mode = MODE_NORMAL;
+really_done:
+  gst_caps_unref (found_caps);
 
   /* 8 */
   if (gst_pad_is_active (pad))
@@ -925,12 +1016,15 @@ gst_type_find_element_change_state (GstElement * element,
   switch (transition) {
     case GST_STATE_CHANGE_PAUSED_TO_READY:
     case GST_STATE_CHANGE_READY_TO_NULL:
+      GST_OBJECT_LOCK (typefind);
       gst_caps_replace (&typefind->caps, NULL);
 
       g_list_foreach (typefind->cached_events,
           (GFunc) gst_mini_object_unref, NULL);
       g_list_free (typefind->cached_events);
       typefind->cached_events = NULL;
+      typefind->mode = MODE_TYPEFIND;
+      GST_OBJECT_UNLOCK (typefind);
       break;
     default:
       break;
